%Sample code for calculating the field diffracted by a square
%aperture and after propagating through an ensemble of spheres. 
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Parameters which may be modified
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

zso = 1.6;                        %Distance from source to beginning of the computational region
zse = 0.3;                        %Distance from beginning of the computational region to
                                  %the detector.
				  
E = 20e3;                         %Photon energy (eV)
Wslit = 20e-6;                    %Side of the  pre-sample square aperture
delta_s = 1e-6;                   %delta of the sphere(s)
beta_s = 0;                       %beta of the sphere(s)
rad_s = 5e-6;                     %Radius of sphere(s)
eta = .0005;                        %Fraction by volume occupied by
                                  %the spheres

z1 = .05;                         %spheres may be located within the region z1<=z<=z2
z2 = .1;
simwidth = 20e-6;                %width of the square which is the
                                  %transverse plane of the
                                  %simulation space
                                  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%End of prameters which may be modified
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			  

lambda = 3e8*4.13566733e-15/E;    %wavelength
V = simwidth^2*(z2-z1)*eta;       %total volume occupied by spheres
Nspher = round(V/(4/3*pi*rad_s^3)); %number of spheres within the volume
fprintf(1,'Number of spheres in simulation space = %d\n',Nspher);

if 1
    [xpos,R] = generate_ensemble(-simwidth/2,simwidth/2,-simwidth/2,simwidth/2,z1,z2,rad_s,rad_s,Nspher);
    save sphere_locations xpos R;
else
    load sphere_locations;
end

%Locations of the sphere centers
xt = xpos(1,:);
yt = xpos(2,:);
zt = xpos(3,:);

%Sort the spheres according to their axial locations
[szt,izt] = sort(zt);

%Vectors of sphere locations sorted according to their axial
%locations
x_s_vec = xt(izt);
y_s_vec = yt(izt);
dz_s_vec = zt(izt);

%The above may be over-written to manually specify the location(s)
%of sphere(s), eg:
%x_s_vec = [0];
%y_s_vec = [7.5e-6];
%dz_s_vec = [0.05];

%The number of sample points in each lateral direction
Ntarget = 2^10*7;
%This ensures that a spherical wave is correctly sampled if
%propagating within the primed coordinate system from z = zso to z=
%zso + zse
dxi_target = sqrt(lambda*(zse)*zso/Ntarget/(zso+ zse)*.95); 

%The cutoff frequency dictated by the choice of sampling period
fco = 1/2/dxi_target*.95;

%This may result in a different number of sample points 
Nxi_lower = round(2*lambda*zse*zso*fco/dxi_target/(zso+ zse));

%This is the actual number of sample points, forced to be a power
%of 2 for numerical efficiency
Nxi = round(2^(log2(Nxi_lower)));

%Half width of xi vector which is used to sample
%the x direction within the sample: subject to constraint
xi_max = Nxi*dxi_target/2; 

%Half width of xi vector which is used to sample
%the y direction within the sample: subject to constraint
yi_max = xi_max; 

%Have the same number of point in the x and y directions
Nyi = Nxi;

%construct xi, yi, fxi and fyi
%xi and yi are the Cartesian coordinates within aperture
%fxi and fyi are the corresponding frequency vectors
dxi = 2*xi_max/(Nxi+1);
dyi = 2*xi_max/(Nxi+1);
xi = (0:(Nxi-1))*dxi;
yi = (0:(Nyi-1))*dyi;

xi = fftshift(xi);
ind0 = find(xi==0);
xi(1:(ind0-1)) = xi(1:(ind0-1)) - xi(ind0-1) - xi(ind0+1);
xi = fftshift(xi);

yi = fftshift(yi);
ind0 = find(yi==0);
yi(1:(ind0-1)) = yi(1:(ind0-1)) - yi(ind0-1) - yi(ind0+1);
yi = fftshift(yi);

fxi = (0:(Nxi-1))/Nxi/dxi;
fxi = fftshift(fxi);
ind0 = find(fxi==0);
fxi(1:(ind0-1)) = fxi(1:(ind0-1)) - fxi(ind0-1) - fxi(ind0+1);
fxi = fftshift(fxi);

fyi = (0:(Nyi-1))/Nyi/dyi;
fyi = fftshift(fyi);
ind0 = find(fyi==0);
fyi(1:(ind0-1)) = fyi(1:(ind0-1)) - fyi(ind0-1) - fyi(ind0+1);
fyi = fftshift(fyi);

%Now we calculate the band limited field diffracted by the
%slits. We are assuming a divergent spherical wave, however, we
%will transform into a plane wave geometry

%Perform numerical integration to evaluate the angular spectrum of
%the wave diffracted by the slit
Ntau = 500;
[tau,wtau] = gauss_legendre(-Wslit/2,Wslit/2,Ntau);

[FXI,TAU] = ndgrid(fxi,tau);
[FXI,WTAU] = ndgrid(fxi,wtau);
[FYI,~] = ndgrid(fyi,wtau);

%Parameter controlling shape of Tukey window
twinparam = 0.25;
%This allows for forcing a portion of the windown in the spectral
%domain to be filled with zeros
fwin_xi= fftshift([zeros(numel(fxi)/4,1);tukeywin(numel(fxi)/2,twinparam);zeros(numel(fxi)/4,1)]);
fwin_yi= fftshift([zeros(numel(fyi)/4,1);tukeywin(numel(fyi)/2,twinparam);zeros(numel(fyi)/4,1)]);

%Perform Fourier transform across x-direction of slit
utilde_fxi = sum( WTAU.*exp(-sqrt(-1)*2*pi*TAU.*FXI), 2);

%Perform Fourier transform across y-direction of slit
utilde_fyi = sum( WTAU.*exp(-sqrt(-1)*2*pi*TAU.*FYI), 2);
			  
%Filtered angular spectrum of the slit
Utilde = (fwin_xi.*utilde_fxi)*(fwin_yi.*utilde_fyi).';

%Uncomment in order to test the use of unfiltered aperture function
%Utilde_unfiltered = (utilde_fxi)*(utilde_fyi).';
%Utilde = Utilde_unfiltered;

%Field immediately after the pre-sample aperture
Ui = ifft2(Utilde)*Nxi*Nyi*fyi(2)*fxi(2);

%Now we need to calculate the angular spectrum of the light
%scattered by a sphere

%This is for integration along the radial coordinate
Nrho = 500;
[rho,wrho] = gauss_legendre(0,rad_s,Nrho);

%We will eventually re-sample this on a Cartesian grid in frequency
%space. We need to make sure that we sample a high enough radial frquency.
f_s = 0:min([fxi(2) fyi(2)]/2):max(sqrt(2)*[fxi fyi]);

[FS,RHO] = ndgrid(f_s,rho);
[FS,WRHO] = ndgrid(f_s,wrho);

%Numerical integration to calculate the spectrum of the field
%scattered by a sphere
utilde_s = 2*pi*sum(WRHO.*(exp(-sqrt(-1)*2*2*pi/lambda*(delta_s-sqrt(-1)*beta_s)*sqrt(rad_s^2-RHO.^2))-1).*besselj(0,2*pi*FS.*RHO).*RHO,2);

%Now resample on the frequency grid used above
Utilde_s = zeros(Nxi,Nyi);
[FXIt,FYIt] = ndgrid(fxi,fyi);

F_s = sqrt(FXIt.^2 + FYIt.^2);

Utilde_s(:) = interp1(f_s,utilde_s,F_s(:),'pchip');

%Add a Dirac delta to make the background of the complex
%transparency equal to 1
Utilde_s(1,1) = Utilde_s(1,1) + 1/(fyi(2)*fxi(2));
[FWXI,FWYI] = ndgrid(fwin_xi,fwin_yi);
FWXI = ones(size(FWXI));
FWYI = ones(size(FWYI));

[XIt,YIt] = ndgrid(xi,yi);

%Calculate the partial magnifications used in the diverging-wave to
%plane-wave transformation 
Mvec = (zso + [dz_s_vec zse])./(zso+[0 dz_s_vec]);

%We are about to start iterating through the multiple slices, set
%some initial values
Ui_z_s = Ui;
M_im1 = 1;

for ispher=1:numel(dz_s_vec)
    x_s = x_s_vec(ispher);
    y_s = y_s_vec(ispher);
    
    %Calculate propagation distance in the primed coordinate system
    %to the next or first sphere
    if ispher==1
	zse_prime = dz_s_vec(ispher)/Mvec(ispher); %Z propagation distance in the primed coordinate system
	
    else
	zse_prime = (dz_s_vec(ispher)-dz_s_vec(ispher-1))/Mvec(ispher);
    end
    %First propagate to the position of the sphere
    %Propagation term
    P = exp(-sqrt(-1)*2*pi/lambda*zse_prime*lambda^2*( (FXIt/M_im1).^2+(FYIt/M_im1).^2)/2).*FWXI.*FWYI; 
    %Apply the propagation term using Fourier theory
    Ui_z_s = ifft2( fft2(Ui_z_s).* P); 
    
    %This is the accumulated magnification
    M_im1 = M_im1*Mvec(ispher);

    %Calculate the frequency domain representation of the sphere's
    %complex transparency
    Utilde_s(:) = interp1(f_s,utilde_s,F_s(:)/M_im1,'pchip')/M_im1^2;

    %Add a Dirac delta to make the background of the complex
    %transparency equal to 1
    Utilde_s(1,1) = Utilde_s(1,1) + 1/(fyi(2)*fxi(2));

    %Calculate the  sphere's transparency function
    %and take account of the transverse positioning of the sphere
    U_s_z_s = ifft2( Utilde_s.*FWXI.*FWYI.*exp(-sqrt(-1)*2*pi*(1/M_im1)*(x_s*FXIt + y_s*FYIt)))*Nxi*Nyi*fyi(2)*fxi(2);

    %Field just after the sphere
    Ui_z_s = U_s_z_s.*Ui_z_s;

    fprintf(1,'Done %d/%d\n',ispher,numel(dz_s_vec));
    
end %end of loop on each sphere

%Now propagate from the last sphere to the observation plane
M2 = Mvec(end);M1 = M_im1;
zse_prime2 = (zse - dz_s_vec(end))/M2;

%Note that our spatial coordinates have been magnified by M1, so
%our frquency coordinates must be de-magnified by the recipricol amount
P2 = exp(-sqrt(-1)*2*pi/lambda*zse_prime2*lambda^2*( (FXIt/M1).^2+(FYIt/M1).^2)/2).*FWXI.*FWYI; %Propagation term
Uobs_prime = ifft2( fft2(Ui_z_s).* P2); %Field in the primed coordinate system
 			       
%Now go back into spherical wave coordinate system
Mtot = prod(Mvec);
Uobs = Uobs_prime.*exp(sqrt(-1)*2*pi/lambda*(zse+zso))/(zso+zse).*exp(sqrt(-1)*2*pi/lambda*( (XIt*Mtot).^2 + (YIt*Mtot).^2 )/2/(zso+zse));

%save('ssevars', 'xi', 'yi', 'Mtot',  'Uobs');



x = fftshift(xi*Mtot);
y = fftshift(yi*Mtot);

figure(1);clf;
imagesc(y*1e6,x*1e6,abs(fftshift(Uobs)));
axis equal;
axis([-1 1 -1 1]*5e-5*1e6);
hx = ylabel('x (\mum)');
hy = xlabel('y (\mum)');

